local ADDON_NAME, ADDON = ...
local MODULE_NAME = "addon"


ADDON[1] = {} -- HST, Addon
ADDON[2] = {} -- C, Config
ADDON[3] = {} -- L, Locale
local CPF, C, L = unpack(ADDON)

--[===[@debug@
_G["HST"] = HST --Expose internals on the Plugin lib
print(ADDON_NAME, MODULE_NAME)
--@end-debug@]===]


---------------------------------------------
-- CONSTANTS
---------------------------------------------
CPF.ADDON_NAME = ADDON_NAME
CPF.VERSION = GetAddOnMetadata(ADDON_NAME, "Version")

CPF.IS_RELEASE_VERSION = CPF.VERSION:match("^%d+.%d+.%d+$") ~= nil
--[===[@debug@
CPF.IS_RELEASE_VERSION = false
--@end-debug@]===]



---------------------------------------------
-- VARIABLES
---------------------------------------------
CPF.isDebug = false


---------------------------------------------
-- METHODS
---------------------------------------------
function CPF:debug(...)
    if ( CPF.isDebug ) then
        print("[" .. CPF.ADDON_NAME .. "]", ...)
    end
end

function CPF:trace(...)
    --@alpha@
    CPF:debug(...)
    --@end-alpha@
end


---------------------------------------------
-- EVENTS & CALLBACKS
---------------------------------------------
CPF.callbacks = LibStub("CallbackHandler-1.0"):New(CPF)
CPF.eventCallbacks = LibStub("CallbackHandler-1.0"):New(CPF, "RegisterEvent", "UnregisterEvent", false)

local frame = CreateFrame("frame")
frame:SetScript("OnEvent", function(self, event, ...)
    CPF.eventCallbacks:Fire(event, ...)
end)

function CPF.eventCallbacks:OnUsed(target, event)
    frame:RegisterEvent(event)
end
function CPF.eventCallbacks:OnUnused(target, event)
    frame:UnregisterEvent(event)
end

local onDemandInitializers = {}
function CPF:InitializeOnDemand(addonName, func)
    if IsAddOnLoaded(addonName) then
        func()
    else
        onDemandInitializers[addonName] = onDemandInitializers[addonName] or {}
        tinsert(onDemandInitializers[addonName], func)
    end
end
CPF.RegisterEvent("onDemandInitializers", "ADDON_LOADED", function(event, addonName)
    if onDemandInitializers[addonName] then
        for _,initializer in pairs(onDemandInitializers[addonName]) do
            initializer()
        end
    end
end)


---------------------------------------------
-- INITIALIZE
---------------------------------------------
--[===[@debug@
LoadAddOn("Blizzard_DebugTools")
--@end-debug@]===]

CPF.RegisterCallback(MODULE_NAME, "initialize", function(event)
end)

CPF.RegisterEvent(MODULE_NAME, "ADDON_LOADED", function(event, addonName)
    if ( addonName == CPF.ADDON_NAME ) then
        CPF.UnregisterEvent(MODULE_NAME, "ADDON_LOADED")

        -- Initialize addon modules
        CPF.callbacks:Fire("initialize")
    end
end)
